# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2017-11-07 15:26
from __future__ import unicode_literals

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djasana', '0009_default_ordering'),
    ]

    operations = [
        migrations.AlterField(
            model_name='webhook',
            name='secret',
            field=models.CharField(max_length=64, validators=[django.core.validators.MinValueValidator(32)]),
        ),
    ]
