# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2017-10-17 14:49
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djasana', '0008_adds_task_blank_null'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='attachment',
            options={'ordering': ('name',)},
        ),
        migrations.AlterModelOptions(
            name='project',
            options={'ordering': ('name',)},
        ),
        migrations.AlterModelOptions(
            name='tag',
            options={'ordering': ('name',)},
        ),
        migrations.AlterModelOptions(
            name='team',
            options={'ordering': ('name',)},
        ),
        migrations.AlterModelOptions(
            name='user',
            options={'ordering': ('name',)},
        ),
        migrations.AlterModelOptions(
            name='workspace',
            options={'ordering': ('name',)},
        ),
    ]
