# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-06-01 14:45
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djasana', '0003_project_allows_blank'),
    ]

    operations = [
        migrations.CreateModel(
            name='SyncToken',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sync', models.CharField(max_length=36)),
                ('project', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='djasana.Project', to_field='remote_id')),
            ],
        ),
        migrations.DeleteModel(
            name='Event',
        ),
    ]
